/* ==================================================== */
/* This source code published as public domain property */ 
/* Developer: Rompo Punyanitya 				   	        */
/*            BCS from RMIT university        	        */
/* Website: www.users.bigpond.net.au/rpunyani/          */
/* Email:   rpunyani@bigpond.net.au                     */
/* ==================================================== */
#ifndef PAGESETUPWINDOW_H
#define PAGESETUPWINDOW_H
#include <Window.h>
#include <Message.h>
#include <PopUpMenu.h>
#include <MenuField.h>
#include <MenuItem.h>
#include <Box.h>
#include <OS.h>
#include <RadioButton.h>
#include <CheckBox.h>
#include <TabView.h>
#include <TextControl.h>
#include <Button.h>
#include <stdio.h>
#include <Application.h>
#include <Alert.h>
#include <Slider.h>

#include <PrinterInfo.h>
#include <PrinterAddon.h>

#include "PageDefs.h"
#include "PageOrderView.h"
#include "PagePrintStyleView.h"
#include "UserDefinePageDlg.h"
#include "GUIUtils.h"
#include "PrintingQualityView.h"
#include "PrintingModeView.h"
#include "ResolutionView.h"

namespace PageSetupWindowSpace
{
  const int DEF_WIDTH	= 416;
  const int DEF_HEIGHT	= 378;
  
  const char DEF_TITLE[]				= "Page Setup";
  
  const char LB_TAB_PAGEFORMAT[] 		= "Page Format";
  const char LB_TAB_OPTION[]			= "Option";
  const char LB_TAB_POSITION[]			= "Position";
  
  const char LB_PAPERSIZE[]				= "Paper size: ";
  const char LB_PAPERFEED[] 			= "Paper feed: ";
  
  const char LB_CUTSHEET[]				= "Cut Sheet Feeder";
  const char LB_PAPERTYPE_EMPTY[]		= "<select>";
   
  // copies
  const char LB_COPIES_GROUP[]			= "Copies";
  const char LB_COPIES[]				= "Copies: ";
  const char LB_COLLATE[]				= "Collate copies";
  const char LB_REVERSE[]				= "Reverse order";
  
  // orientation
  const char LB_ORIENTATION_GROUP[]		= "Orientation";
  const char LB_PORTRAIT[]				= "Portrait";
  const char LB_LANDSCAPE[]				= "Landscape";
  const char LB_180ROTATE[]				= "Rotate 180 degree";
  
  // printable area
  const char LB_PRINTABLEAREA_GROUP[]	= "Printable area";
  const char LB_STANDARD[]				= "Standard";
  const char LB_CENTERED[]				= "Centered";

  // option
  const char LB_PAPERTYPE_GROUP[]			= "Paper";
  const char LB_PAPERTYPE[]					= "Paper: ";
  
  const char LB_MODE_COLOR_GROUP[]			= "Color";
  
  const char LB_MODE_COLOR_COLOR[]			= "Color";
  const char LB_MODE_COLOR_MONO[]			= "Monochrome";

  const char LB_QUALITY_GROUP[]				= "Mode";
  const char LB_QUALITY[]					= "Quality: ";
  const char LB_MIN_QUALITY[]				= "Poor";
  const char LB_MAX_QUALITY[]				= "Best";
    
  // Buttons
  const char LB_DISCARD[]				= "Discard";
  const char LB_REVERT[]				= "Revert";
  const char LB_SAVE[]					= "Save";
    
  const uint32 MSGCODE_COLLATE_ORDER	= 'MSG0';
  const uint32 MSGCODE_REVERSE_ORDER	= 'MSG0' + 1;
  const uint32 MSGCODE_PORTRAIT			= 'MSG0' + 2;
  const uint32 MSGCODE_LANDSCAPE		= 'MSG0' + 3;
  const uint32 MSGCODE_STANDARD			= 'MSG0' + 4;
  const uint32 MSGCODE_CENTERED			= 'MSG0' + 5;
  const uint32 MSGCODE_PAPERSIZE		= 'MSG0' + 6;

  const uint32 MSGCODE_DISCARD			= 'MSG0' + 7;
  const uint32 MSGCODE_REVERT			= 'MSG0' + 8;
  const uint32 MSGCODE_SAVE				= 'MSG0' + 9;
  const uint32 MSGCODE_NUM_OF_COPIES	= 'MSG0' + 10;
  const uint32 MSGCODE_USERDEFINEPAGEDLG_RESULT		= 'MSG0' + 11;
  
  const uint32 MSGCODE_PAPERTYPE			= 'msg0' + 12;
  const uint32 MSGCODE_COLOR				= 'msg0' + 13;
  const uint32 MSGCODE_QUALITY_SCALE		= 'msg0' + 14;  
};
class PageSetupWindow : public BWindow
{
  public:
    PageSetupWindow(PrinterAddon *,uint8 unit = 0);
    virtual void Quit();
    virtual void MessageReceived(BMessage *);
    void Go(BMessage *);
  private:
    void BuildGUI();
    void LoadPaperFeederList();
    
    void OnOrderChange();
    void OnStyleChange();
    
    void OnDiscard();							// done
    void OnRevert();							// done
    void OnSave();								// done
    void OnNumberOfCopiesChange();				// done
    void OnUserDefinePaper();					// done
    void OnUserDefinePageDlgResult(BMessage *);	// done

    void OnEventPaperType();					// not
    void OnEventColorMode();					// not
    void OnEventQualityScale();					// not

    void Retrieve    (BMessage *);				// yes
    void SetTo       (BMessage *);				// yes 
    
    int32 GetPaperSize();				// yes // return paper kind
    float GetUserDefinePaperWidth();	// yes // return user define paper width
    float GetUserDefinePaperHeight();	// yes // return user define paper height
    int32 GetNumberOfCopies();			// yes // return number of copies
    int32 GetPaperFeeder();				// no  // return id of paper feeder
    
    void UpdateUserDefinePageSizeMenuLabel();	// done
    void ReselectPageSizeMenu();				// done

    void LoadPaperType();						// yes
    bool IsGUIColorMode();						// yes
    int32 GetGUISelectPaperType();				// yes
    void SetSDQualityFromGUI();					// yes
    void CreateSDQuality(int value,int count);  // yes
    void SetLBQualityFromGUI();					// not
    void SetBVQualityFromGUI();					// not
  private:
    PrinterAddon *faddon;
    sem_id   fsem;
    BMessage *fmsg;
    float fuserdefine_width;
    float fuserdefine_height;
    int32 fpaper_size;
    uint8 funit;
  private:
    BTabView   			*tb_view;
    BPopUpMenu 			*mn_paper;
    BPopUpMenu 			*mn_feed;
    PagePrintStyleView	*bv_style;
    PageOrderView 		*bv_copies;
    BTextControl 		*tf_copies;
    BCheckBox    		*chk_collate;
    BCheckBox    		*chk_reverse;
    BRadioButton 		*rb_portrait;
    BRadioButton 		*rb_landscape;
    BCheckBox    		*chk_180rotate;
    BRadioButton 		*rb_parea_standard;
    BRadioButton 		*rb_parea_centered;
    BButton      		*btn_discard;
    BButton      		*btn_revert;
    BButton      		*btn_save;
    BMenuItem	 		*mn_userdefine_paper_item;
    UserDefinePageDlg	*userdlg;

    // Option 
    BPopUpMenu			*mn_paper_type;
    PrintingModeView	*bv_color_mode;
    BRadioButton		*rb_mode_color;
    BRadioButton		*rb_mode_mono;
    BBox				*f_quality_box;
    BRect        		f_quality_rect;
    PrintingQualityView *bv_quality;
    BSlider				*sd_quality;
    ResolutionView		*lb_resolution;
};
#endif